library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_arith.all;
use IEEE.std_logic_unsigned.all;

entity effector_TB is
end effector_TB;


architecture rtl of effector_TB is

component effector is
port (
	clk : in std_logic;
	audio_clk : in std_logic;
	reset_n : in std_logic;

	-- for temporal testing... we will include this as bus commande from CPU!
	distortion_enable : in std_logic;
	clip : in std_logic_vector(15 downto 0);	

    vibrato_enable: in std_logic;
    delay_amount: in std_logic_vector(2 downto 0);

    -- Audio interface signals
	AUD_ADCLRCK  : out std_logic;   --    Audio CODEC ADC LR Clock
    AUD_ADCDAT   : in  std_logic;   --    Audio CODEC ADC Data
    AUD_BCLK     : inout std_logic;  --    Audio CODEC Bit-Stream Clock
    AUD_DACLRCK  : out std_logic;   --    Audio CODEC DAC LR Clock
    AUD_DACDAT   : out std_logic   --    Audio CODEC DAC Data
);
end component;

signal	clk : std_logic;
signal	audio_clk : std_logic;
signal	reset_n : std_logic;

signal	distortion_enable : std_logic;
signal	clip : std_logic_vector(15 downto 0);	

signal    vibrato_enable: std_logic;
signal    delay_amount: std_logic_vector(2 downto 0);

signal	AUD_ADCLRCK  : std_logic;   --    Audio CODEC ADC LR Clock
signal    AUD_ADCDAT   : std_logic;   --    Audio CODEC ADC Data
signal    AUD_BCLK     : std_logic;  --    Audio CODEC Bit-Stream Clock
signal    AUD_DACLRCK  : std_logic;   --    Audio CODEC DAC LR Clock
signal    AUD_DACDAT   : std_logic;   --    Audio CODEC DAC Data

begin

 E1: effector port map(
	clk => clk,
	audio_clk => audio_clk,
	reset_n => reset_n,

	-- for temporal testing... we will include this as bus commande from CPU!
	distortion_enable => distortion_enable,
	clip => clip,

    vibrato_enable => vibrato_enable,
    delay_amount => delay_amount,

    -- Audio interface signals
	AUD_ADCLRCK  => AUD_ADCLRCK,
    AUD_ADCDAT   => AUD_ADCDAT,
    AUD_BCLK     => AUD_BCLK,
    AUD_DACLRCK  => AUD_DACLRCK,
    AUD_DACDAT   => AUD_DACDAT
);



	process
	begin
		clk <='0';
		reset_n <= '0';
		wait for 5 ns;
		clk <= '1';
		wait for 5 ns;
		clk <='0';
		reset_n <= '1';
		wait for 5 ns;
		loop
			clk <='1';
			wait for 5 ns;
			clk <='0';
			wait for 5 ns;
		end loop;
	end process;

	process
	begin
		audio_clk <='0';
		wait for 40 ns;
		audio_clk <= '1';
		wait for 40 ns;
		loop
			audio_clk <='1';
			wait for 40 ns;
			audio_clk <='0';
			wait for 40 ns;
		end loop;
	end process;

	process
	begin
		distortion_enable <= '0';
		vibrato_enable <= '1';
		delay_amount <= "100";
		loop
		-- 1010 1110 1111 0010
		AUD_ADCDAT <= '1'; -- 1
		wait for 120 ns;
		AUD_ADCDAT <= '0'; -- 2
		wait for 120 ns;
		AUD_ADCDAT <= '1'; -- 3
		wait for 120 ns;
		AUD_ADCDAT <= '0'; -- 4
		wait for 120 ns;
		AUD_ADCDAT <= '1'; -- 5
		wait for 120 ns;
		AUD_ADCDAT <= '1'; -- 6
		wait for 120 ns;
		AUD_ADCDAT <= '1'; -- 7
		wait for 120 ns;
		AUD_ADCDAT <= '0'; -- 8
		wait for 120 ns;
		AUD_ADCDAT <= '1'; -- 9
		wait for 120 ns;
		AUD_ADCDAT <= '1'; -- 10
		wait for 120 ns;
		AUD_ADCDAT <= '1'; -- 11
		wait for 120 ns;
		AUD_ADCDAT <= '1'; -- 12
		wait for 120 ns;
		AUD_ADCDAT <= '0'; -- 13
		wait for 120 ns;
		AUD_ADCDAT <= '0'; -- 14
		wait for 120 ns;
		AUD_ADCDAT <= '1'; -- 15
		wait for 120 ns;
		AUD_ADCDAT <= '0'; -- 16
		wait for 120 ns;
		AUD_ADCDAT <= '1'; -- 1
		wait for 120 ns;
		AUD_ADCDAT <= '1'; -- 2
		wait for 120 ns;
		AUD_ADCDAT <= '1'; -- 3
		wait for 120 ns;
		AUD_ADCDAT <= '1'; -- 4
		wait for 120 ns;
		AUD_ADCDAT <= '0'; -- 5
		wait for 120 ns;
		AUD_ADCDAT <= '0'; -- 6
		wait for 120 ns;
		AUD_ADCDAT <= '0'; -- 7
		wait for 120 ns;
		AUD_ADCDAT <= '0'; -- 8
		wait for 120 ns;
		AUD_ADCDAT <= '1'; -- 9
		wait for 120 ns;
		AUD_ADCDAT <= '1'; -- 10
		wait for 120 ns;
		AUD_ADCDAT <= '1'; -- 11
		wait for 120 ns;
		AUD_ADCDAT <= '1'; -- 12
		wait for 120 ns;
		AUD_ADCDAT <= '0'; -- 13
		wait for 120 ns;
		AUD_ADCDAT <= '0'; -- 14
		wait for 120 ns;
		AUD_ADCDAT <= '0'; -- 15
		wait for 120 ns;
		AUD_ADCDAT <= '0'; -- 16
		end loop;	
	end process;		

end architecture;